/**
	@file
	 ,    
	   M8C (Cypress).  
	     8051.
	
	2009.07.25
		 -    .  
		  ,   .  ,   
		 ,   .  ,   
		   .      
		  256 . Flash-   65.
		
		      ,   
		     .
		
	2009.07.28
		  F     C  F.   
		   .   MVI.
		         
		-.
		
	2009.07.29
		   (SBB, ADC,  SUB).   
		      REG-.  
		 DEFINE   .
		    .
	
	@todo (+)
	  C/Z/I/...    F (Flags),   
	   M8C.
	
	@todo (+)
	   ,   .
	
	@todo
	      M8C,   
	           
	 .
	
	@todo
	,       
	  .
	
	@todo 
	    .      
	  ,        
	. 
	
	@todo (+)
	       define-.
	
	@todo 
	        (REG)?   
	,       ,   
	    .
	
	@todo (!)
	    . ,    
	,          
	       ( HALT).
	
	@todo (+)
	   ,   REG- 
	  DEFINE.
	
	@todo
	    .  ,   
	      h- (cy803.h,  ).
	
	igorkov / igorkov.rus@gmail.com / 2009
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "cy803.h"

uint8_t ram_img[EMU_RAM_SIZE][EMU_RAM_BANKS];

#if defined( EMU_REG )
	uint8_t ram_reg[EMU_RAM_SIZE][2];
#endif

      uint8_t *ram;
const uint8_t *rom;

#ifdef DLL_BUILD
	#define DLL_EXPORT __declspec(dllexport)
#else
	#define DLL_EXPORT
#endif

uint8_t A;
uint8_t X;
uint8_t SP;
//uint8_t C;
//uint8_t Z;
uint8_t F;
uint16_t PC;

#define FLAG_XIO	0x10
#define FLAG_C		0x04
#define FLAG_Z		0x02
#define FLAG_GIE	0x01
#define FLAG_PGMODE	0xC0

#define IS_Z()		(F&FLAG_Z?1:0)
#define IS_C()		(F&FLAG_C?1:0)
#define IS_XIO()	(F&FLAG_XIO?1:0)
#define IS_GIE()	(F&FLAG_GIE?1:0)

#define GET_PGMODE()	((F& 0xC0)>>6)
#define SET_PGMODE(a)	((F&~0xC0)|(a<<6))

#define SET_Z()		{F|=FLAG_Z;}
#define SET_C()		{F|=FLAG_C;}
#define SET_XIO()	{F|=FLAG_XIO;}
#define SET_GIO()	{F|=FLAG_GIO;}

#define CLR_Z()		{F&=~FLAG_Z;}
#define CLR_C()		{F&=~FLAG_C;}
#define CLR_XIO()	{F&=~FLAG_XIO;}
#define CLR_GIO()	{F&=~FLAG_GIO;}

#if defined(DEBUG_EMU)

	#define DBG_EMU(n, a ...) 			\
		{								\
			char s[256], p[10];			\
			sprintf(s, a);				\
			sprintf(p, "%%-%ds", n);	\
			printf(p, s);				\
		}
#else
	#define DBG_EMU(n, a ...) 
#endif

#define INS()			(rom[PC])
#define DT1()			(rom[PC+1])
#define DT2()			(rom[PC+2])
#define DT3()			(rom[PC+3])
#define DT4()			(rom[PC+4])

#define JMP_OFFSET()	(((INS()&0x08)?-0x1000:0)+((((uint16_t)INS()&0x0F)<<8)|DT1()))
#define ABS_ADDR()		(rom[PC+2] | (rom[PC+1] << 8))

#define EXPR1()			(rom[PC+1])
#define EXPR2()			(rom[PC+1])

#define PEXPR1()		(ram[EXPR1()])
#define PEXPR1X()		(ram[(uint8_t)(EXPR1() + X)])

#define PEXPR2()		(ram[EXPR2()])
#define PEXPR2X()		(ram[(uint8_t)(EXPR2() + X)])

#define HB(a)			((a&0xFF00)>>8)
#define LB(a)			 (a&0x00FF)

#define PUSH(x)			(ram[x++])
#define POP(x)			(ram[--x])

#define READ_ROM(x)		(rom[(uint16_t)x])
#define READ_RAM(x)		(ram[(uint8_t)x])
#define READ_REG(x)		(ram_reg[(uint8_t)x][(IS_XIO()?1:0)])

#if defined(DEBUG_EMU)
int step_counter = 0;
int cy803_dbg_getsteps(void)
{
	return step_counter;
}
#endif

DLL_EXPORT int cy803_init(uint8_t *romaddr)
{
	rom = romaddr;
	ram = ram_img[0];

	memset(ram, 0x00, EMU_RAM_SIZE);

	A  = 0;
	X  = 0;
	SP = EMU_RAM_SIZE / 2;
	PC = 0;
	F  = 0;
	
#if defined(DEBUG_EMU)
	step_counter = 0;
#endif
	return VMRET_OK;
}

DLL_EXPORT int cy803_ccall(uint16_t addr, uint8_t argc, uint8_t *argv, uint8_t retc, uint8_t *retv)
{
	int reti;
	int i = 0;
	
	ram = ram_img[0];
	
	while (argc != i)
	{
		PUSH(SP) = argv[i];
		i++;
	}
	
	PC = addr;
	
	while ((reti = cy803_step()) == 0);
	
	if (reti != 5)
	{
		return reti;
	}

	i = 0;
	while (argc != i)
	{
		retv[i] = POP(SP);
		i++;
	}

	return VMRET_OK;
}

DLL_EXPORT int cy803_step(void)
{
#if defined(DEBUG_EMU)
	step_counter++;
#endif
	DBG_EMU(31, "[STEP=%04d | OPCODE=%02x]  %04x: ", step_counter, INS(), PC);
	
	switch (INS() & 0xF0)
	{
	/*
		Opcode Hex | Cycles | Bytes | Instruction Format | Flags
		8x           5        2       JMP
	 */
	case 0x80:
		DBG_EMU(20, "JMP %4d  ", JMP_OFFSET());
		PC = (PC+1) + JMP_OFFSET();
		break;
		
	/*
		Opcode Hex | Cycles | Bytes | Instruction Format | Flags
		9x           11       2       CALL
	 */
	case 0x90:
		DBG_EMU(20, "CALL %d ", JMP_OFFSET());
		PUSH(SP) = HB(PC+2);
		PUSH(SP) = LB(PC+2);
		PC = (PC+2) + JMP_OFFSET();
		break;
	
	/*
		Opcode Hex | Cycles | Bytes | Instruction Format | Flags
		Ax           5        2       JZ
		Bx           5        2       JNZ
		Cx           5        2       JC
		Dx           5        2       JNC
		Ex           7        2       JACC
	*/
	case 0xA0:
		DBG_EMU(20, "JZ %d ", JMP_OFFSET());
		if (IS_Z()) PC = (PC + 1) + JMP_OFFSET(); else PC += 2;
		break;
	case 0xB0:
		DBG_EMU(20, "JNZ %d ", JMP_OFFSET());
		if (!IS_Z()) PC = (PC + 1) + JMP_OFFSET(); else PC += 2;
		break;
	case 0xC0:
		DBG_EMU(20, "JC %d ", JMP_OFFSET());
		if (IS_C()) PC = (PC + 1) + JMP_OFFSET(); else PC += 2;
		break;
	case 0xD0:
		DBG_EMU(20, "JNC %d ", JMP_OFFSET());
		if (!IS_C()) PC = (PC + 1) + JMP_OFFSET(); else PC += 2;
		break;
	case 0xE0:
		DBG_EMU(20, "JACC %d ", JMP_OFFSET());
		// PC = (PC + 1) + k + A
		PC = (PC + 1) + JMP_OFFSET() + A;
		break;
		
	/*
		Opcode Hex | Cycles | Bytes | Instruction Format | Flags
		Fx           13       2       INDEX                Z
	 */
	case 0xF0:
		DBG_EMU(20, "INDEX ");
		// A = rom[k + A + PC + 2], (2048 <= k <= 2047)
		A = READ_ROM((uint16_t)JMP_OFFSET() + A + PC + 2);
		PC += 2;
		break;

	default:
		switch (INS())
		{
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			00           15       1       SSC 
		 */
		case 0x00:
			DBG_EMU(20, "SSC "); // System Call
#if 1
			DBG_EMU(15, "| Return VMRET_SSC;");
			DBG_EMU(0, "\n");
			return VMRET_SSC;
#endif
			PC += 1;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			01           4        2       ADD A, expr          C, Z 
			02           6        2       ADD A, [expr]        C, Z 
			03           7        2       ADD A, [X+expr]      C, Z 
			04           7        2       ADD [expr], A        C, Z 
			05           8        2       ADD [X+expr], A      C, Z 
			06           9        3       ADD [expr], expr     C, Z 
			07           10       3       ADD [X+expr], expr   C, Z 
		*/
		case 0x01:
			DBG_EMU(20, "ADD A, #%02x ", EXPR1());
			if ((uint8_t)(A+EXPR1()) == 0) SET_Z() else CLR_Z();
			if ((uint16_t)A+EXPR1() > 255) SET_C() else CLR_C();
			A += EXPR1();
			PC += 2;
			break;
		case 0x02:
			DBG_EMU(20, "ADD A, [%d] ", EXPR1());
			if ((uint8_t)(A+PEXPR1()) == 0) SET_Z() else CLR_Z();
			if ((uint16_t)A+PEXPR1() > 255) SET_C() else CLR_C();
			A += PEXPR1();
			PC += 2;
			break;
		case 0x03:
			DBG_EMU(20, "ADD A, [%d+X] ", EXPR1());
			if ((uint8_t)(A+PEXPR1X()) == 0) SET_Z() else CLR_Z();
			if ((uint16_t)A+PEXPR1X() > 255) SET_C() else CLR_C();
			A += PEXPR1X();
			PC += 2;
			break;
		case 0x04:
			DBG_EMU(20, "ADD [%d], A ", EXPR1());
			if ((uint8_t)(A+PEXPR1()) == 0) SET_Z() else CLR_Z();
			if ((uint16_t)A+PEXPR1() > 255) SET_C() else CLR_C();
			PEXPR1() += A;
			PC += 2;
			break;
		case 0x05:
			DBG_EMU(20, "ADD [%d+X], A ", EXPR1());
			if ((uint8_t)(A+PEXPR1X()) == 0) SET_Z() else CLR_Z();
			if ((uint16_t)A+PEXPR1X() > 255) SET_C() else CLR_C();
			PEXPR1X() += A;
			PC += 2;
			break;
		case 0x06:
			DBG_EMU(20, "ADD [%d], %d ", EXPR1(), EXPR2());
			if ((uint8_t)(PEXPR1()+EXPR2()) == 0) SET_Z() else CLR_Z();
			if ((uint16_t)PEXPR1()+EXPR2() > 255) SET_C() else CLR_C();
			PEXPR1() += EXPR2();
			PC += 3;
			break;
		case 0x07:
			DBG_EMU(20, "ADD [%d+X], %d ", EXPR1(), EXPR2());
			if ((uint8_t)(PEXPR1X()+EXPR2()) == 0) SET_Z() else CLR_Z();
			if ((uint16_t)PEXPR1X()+EXPR2() > 255) SET_C() else CLR_C();
			PEXPR1X() += EXPR2();
			PC += 3;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			08           4        1       PUSH A 
		 */
		case 0x08:
			DBG_EMU(20, "PUSH A     ");
			PUSH(A);
			PC += 1;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			09           4        2       ADC A, expr          C, Z 
			0A           6        2       ADC A, [expr]        C, Z 
			0B           7        2       ADC A, [X+expr]      C, Z 
			0C           7        2       ADC [expr], A        C, Z 
			0D           8        2       ADC [X+expr], A      C, Z 
			0E           9        3       ADC [expr], expr     C, Z 
			0F           10       3       ADC [X+expr], expr   C, Z 
		*/
		case 0x09:
			DBG_EMU(20, "ADC [%d+X], %d ", EXPR1(), EXPR2());
			if ((uint8_t)(A+EXPR1()+IS_C()) == 0) SET_Z() else CLR_Z();
			if ((uint16_t)A+EXPR1()+IS_C() > 255) SET_C() else CLR_C();
			A += EXPR1() + IS_C();
			PC += 2;
			break;
		case 0x0A:
			DBG_EMU(20, "ADC A, [%d] ", EXPR1());
			if ((uint8_t)(A+PEXPR1()+IS_C()) == 0) SET_Z() else CLR_Z();
			if ((uint16_t)A+PEXPR1()+IS_C() > 255) SET_C() else CLR_C();
			A += PEXPR1() + IS_C();
			PC += 2;
			break;
		case 0x0B:
			DBG_EMU(20, "ADC A, [%d+X] ", EXPR1());
			if ((uint8_t)(A+PEXPR1X()+IS_C()) == 0) SET_Z() else CLR_Z();
			if ((uint16_t)A+PEXPR1X()+IS_C() > 255) SET_C() else CLR_C();
			A += PEXPR1X() + IS_C();
			PC += 2;
			break;
		case 0x0C:
			DBG_EMU(20, "ADC [%d], A ", EXPR1());
			if ((uint8_t)(A+PEXPR1()+IS_C()) == 0) SET_Z() else CLR_Z();
			if ((uint16_t)A+PEXPR1()+IS_C() > 255) SET_C() else CLR_C();
			PEXPR1() += A + IS_C();
			PC += 2;
			break;
		case 0x0D:
			DBG_EMU(20, "ADC [%d+X], A ", EXPR1());
			if ((uint8_t)(A+PEXPR1X()+IS_C()) == 0) SET_Z() else CLR_Z();
			if ((uint16_t)A+PEXPR1X()+IS_C() > 255) SET_C() else CLR_C();
			PEXPR1X() += A + IS_C();
			PC += 2;
			break;
		case 0x0E:
			DBG_EMU(20, "ADC [%d], %d ", EXPR1(), EXPR2());
			if ((uint8_t)(PEXPR1()+EXPR2()+IS_C()) == 0) SET_Z() else CLR_Z();
			if ((uint16_t)PEXPR1()+EXPR2()+IS_C() > 255) SET_C() else CLR_C();
			PEXPR1() += EXPR2() + IS_C();
			PC += 3;
			break;
		case 0x0F:
			DBG_EMU(20, "ADC [%d+X], %d ", EXPR1(), EXPR2());
			if ((uint8_t)(PEXPR1X()+EXPR2()+IS_C()) == 0) SET_Z() else CLR_Z();
			if ((uint16_t)PEXPR1X()+EXPR2()+IS_C() > 255) SET_C() else CLR_C();
			PEXPR1X() += EXPR2() + IS_C();
			PC += 3;
			break;		
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			10           4        1       PUSH X 
		 */
		case 0x10:
			DBG_EMU(20, "PUSH X     ");
			PUSH(X);
			PC += 1;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			11           4        2       SUB A, expr          C, Z 
			12           6        2       SUB A, [expr]        C, Z 
			13           7        2       SUB A, [X+expr]      C, Z 
			14           7        2       SUB [expr], A        C, Z 
			15           8        2       SUB [X+expr], A      C, Z 
			16           9        3       SUB [expr], expr     C, Z 
			17           10       3       SUB [X+expr], expr   C, Z 
		*/
		case 0x11:
			DBG_EMU(20, "SUB A, %d ", EXPR1());
			if (A < EXPR1()) SET_C() else CLR_C();
			if (A == EXPR1()) SET_Z() else CLR_Z();
			A -= EXPR1();
			PC += 2;
			break;
		case 0x12:
			DBG_EMU(20, "SUB A, [%d] ", EXPR1());
			if (A < PEXPR1()) SET_C() else CLR_C();
			if (A == PEXPR1()) SET_Z() else CLR_Z();
			A -= PEXPR1();
			PC += 2;
			break;
		case 0x13:
			DBG_EMU(20, "SUB A, [%d+X] ", EXPR1());
			if (A < PEXPR1X()) SET_C() else CLR_C();
			if (A == PEXPR1X()) SET_Z() else CLR_Z();
			A -= PEXPR1X();
			PC += 2;
			break;
		case 0x14:
			DBG_EMU(20, "SUB [%d], A ", EXPR1());
			if (PEXPR1() < A) SET_C() else CLR_C();
			if (PEXPR1() == A) SET_Z() else CLR_Z();
			PEXPR1() -= A;
			PC += 2;
			break;
		case 0x15:
			DBG_EMU(20, "SUB [%d+X], A ", EXPR1());
			if (PEXPR1X() < A) SET_C() else CLR_C();
			if (PEXPR1X() == A) SET_Z() else CLR_Z();
			PEXPR1X() -= A;
			PC += 2;
			break;
		case 0x16:
			DBG_EMU(20, "SUB [%d], %d ", EXPR1(), EXPR2());
			if (PEXPR1() < EXPR2()) SET_C() else CLR_C();
			if (PEXPR1() == EXPR2()) SET_Z() else CLR_Z();
			PEXPR1() -= EXPR2();
			PC += 3;
			break;
		case 0x17:
			DBG_EMU(20, "SUB [%d+X], %d ", EXPR1(), EXPR2());
			if (PEXPR1X() < EXPR2()) SET_C() else CLR_C();
			if (PEXPR1X() == EXPR2()) SET_Z() else CLR_Z();
			PEXPR1X() -= EXPR2();
			PC += 3;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			18           5        1       POP A                Z 
		 */
		case 0x18:
			DBG_EMU(20, "POP A      ");
			POP(A);
			if (A == 0) SET_Z() else CLR_Z();
			PC += 1;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			19           4        2       SBB A, expr          C, Z 
			1A           6        2       SBB A, [expr]        C, Z 
			1B           7        2       SBB A, [X+expr]      C, Z 
			1C           7        2       SBB [expr], A        C, Z 
			1D           8        2       SBB [X+expr], A      C, Z 
			1E           9        3       SBB [expr], expr     C, Z 
			1F           10       3       SBB [X+expr], expr   C, Z 
		 */
		case 0x19:
			DBG_EMU(20, "SBB [%d+X], %d ", EXPR1(), EXPR2());
			if (A==(uint8_t)EXPR1()+IS_C()) SET_Z() else CLR_Z();
			if (A < (uint16_t)EXPR1()+IS_C()) SET_C() else CLR_C();
			A -= EXPR1() + IS_C();
			PC += 2;
			break;
		case 0x1A:
			DBG_EMU(20, "SBB A, [%d] ", EXPR1());
			if (A==(uint8_t)PEXPR1()+IS_C()) SET_Z() else CLR_Z();
			if (A < (uint16_t)PEXPR1()+IS_C()) SET_C() else CLR_C();
			A -= PEXPR1() + IS_C();
			PC += 2;
			break;
		case 0x1B:
			DBG_EMU(20, "SBB A, [%d+X] ", EXPR1());
			if (A==(uint8_t)PEXPR1X()+IS_C()) SET_Z() else CLR_Z();
			if (A < (uint16_t)PEXPR1X()+IS_C()) SET_C() else CLR_C();
			A -= PEXPR1X() + IS_C();
			PC += 2;
			break;
		case 0x1C:
			DBG_EMU(20, "SBB [%d], A ", EXPR1());
			if (PEXPR1()==(uint8_t)A+IS_C()) SET_Z() else CLR_Z();
			if (PEXPR1() < (uint16_t)A+IS_C()) SET_C() else CLR_C();
			A -= PEXPR1() + IS_C();
			PC += 2;
			break;
		case 0x1D:
			DBG_EMU(20, "SBB [%d+X], A ", EXPR1());
			if (PEXPR1X()==(uint8_t)A+IS_C()) SET_Z() else CLR_Z();
			if (PEXPR1X() < (uint16_t)A+IS_C()) SET_C() else CLR_C();
			A -= PEXPR1X() + IS_C();
			PC += 2;
			break;
		case 0x1E:
			DBG_EMU(20, "SBB [%d], %d ", EXPR1(), EXPR2());
			if (PEXPR1()==(uint8_t)EXPR2()+IS_C()) SET_Z() else CLR_Z();
			if (PEXPR1() < (uint16_t)EXPR2()+IS_C()) SET_C() else CLR_C();
			PEXPR1() -= EXPR2() + IS_C();
			PC += 3;
			break;
		case 0x1F:
			DBG_EMU(20, "SBB [%d+X], %d ", EXPR1(), EXPR2());
			if (PEXPR1X()==(uint8_t)EXPR2()+IS_C()) SET_Z() else CLR_Z();
			if (PEXPR1X() < (uint16_t)EXPR2()+IS_C()) SET_C() else CLR_C();
			PEXPR1X() -= EXPR2() + IS_C();
			PC += 3;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			20           5        1       POP X 
		 */
		case 0x20:
			DBG_EMU(20, "POP X      ");
			POP(X);
			PC += 1;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			21           4        2       AND A, expr          Z 
			22           6        2       AND A, [expr]        Z 
			23           7        2       AND A, [X+expr]      Z 
			24           7        2       AND [expr], A        Z 
			25           8        2       AND [X+expr], A      Z 
			26           9        3       AND [expr], expr     Z 
			27           10       3       AND [X+expr], expr   Z 
		 */
		case 0x21:
			DBG_EMU(20, "AND [%d+X], %d ", EXPR1(), EXPR2());
			A &= EXPR1();
			if (A==0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		case 0x22:
			DBG_EMU(20, "AND A, [%d] ", EXPR1());
			A &= PEXPR1();
			if (A==0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		case 0x23:
			DBG_EMU(20, "AND A, [%d+X] ", EXPR1());
			A &= PEXPR1X();
			if (A==0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		case 0x24:
			DBG_EMU(20, "AND [%d], A ", EXPR1());
			PEXPR1() &= A;
			if (PEXPR1()==0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		case 0x25:
			DBG_EMU(20, "AND [%d+X], A ", EXPR1());
			PEXPR1X() &= A;
			if (PEXPR1X()==0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		case 0x26:
			DBG_EMU(20, "AND [%d], %d ", EXPR1(), EXPR2());
			PEXPR1() &= EXPR2();
			if (PEXPR1()==0) SET_Z() else CLR_Z();
			PC += 3;
			break;
		case 0x27:
			DBG_EMU(20, "AND [%d+X], %d ", EXPR1(), EXPR2());
			PEXPR1X() &= EXPR2();
			if (PEXPR1X()==0) SET_Z() else CLR_Z();
			PC += 3;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			28           11       1       ROMX                 Z 
		 */
		case 0x28:
			DBG_EMU(20, "ROMX ");
			A = READ_ROM(((uint8_t)A<<8)|X);
			PC += 1;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			29           4        2       OR A, expr           Z 
			2A           6        2       OR A, [expr]         Z 
			2B           7        2       OR A, [X+expr]       Z 
			2C           7        2       OR [expr], A         Z 
			2D           8        2       OR [X+expr], A       Z 
			2E           9        3       OR [expr], expr      Z 
			2F           10       3       OR [X+expr], expr    Z 
		 */   
		case 0x29:
			DBG_EMU(20, "OR [%d+X], %d ", EXPR1(), EXPR2());
			A |= EXPR1();
			if (A==0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		case 0x2A:
			DBG_EMU(20, "OR A, [%d] ", EXPR1());
			A |= PEXPR1();
			if (A==0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		case 0x2B:
			DBG_EMU(20, "OR A, [%d+X] ", EXPR1());
			A |= PEXPR1X();
			if (A==0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		case 0x2C:
			DBG_EMU(20, "OR [%d], A ", EXPR1());
			PEXPR1() |= A;
			if (PEXPR1() == 0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		case 0x2D:
			DBG_EMU(20, "OR [%d+X], A ", EXPR1());
			PEXPR1X() |= A;
			if (PEXPR1X() == 0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		case 0x2E:
			DBG_EMU(20, "OR [%d], %d ", EXPR1(), EXPR2());
			PEXPR1() |= PEXPR2();
			if (PEXPR1() == 0) SET_Z() else CLR_Z();
			PC += 3;
			break;
		case 0x2F:
			DBG_EMU(20, "OR [%d+X], %d ", EXPR1(), EXPR2());
			PEXPR1X() |= PEXPR2();
			if (PEXPR1X() == 0) SET_Z() else CLR_Z();
			PC += 3;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			30           9        1       HALT 
		 */
		case 0x30:
			DBG_EMU(20, "HALT ");
			DBG_EMU(15, "| Return VMRET_HALT;");
			DBG_EMU(0, "\n");
			PC += 1;
			return VMRET_HALT;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			31           4        2       XOR A, expr          Z 
			32           6        2       XOR A, [expr]        Z 
			33           7        2       XOR A, [X+expr]      Z 
			34           7        2       XOR [expr], A        Z 
			35           8        2       XOR [X+expr], A      Z 
			36           9        3       XOR [expr], expr     Z 
			37           10       3       XOR [X+expr], expr   Z 
		 */                       
		case 0x31:
			DBG_EMU(20, "XOR [%d+X], %d ", EXPR1(), EXPR2());
			A ^= EXPR1();
			if (A==0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		case 0x32:
			DBG_EMU(20, "XOR A, [%d] ", EXPR1());
			A ^= PEXPR1();
			if (A==0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		case 0x33:
			DBG_EMU(20, "XOR A, [%d+X] ", EXPR1());
			A ^= PEXPR1X();
			if (A==0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		case 0x34:
			DBG_EMU(20, "XOR [%d], A ", EXPR1());
			PEXPR1() ^= A;
			if (A^PEXPR1() == 0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		case 0x35:
			DBG_EMU(20, "XOR [%d+X], A ", EXPR1());
			PEXPR1X() ^= EXPR2();
			if (A^PEXPR1X() == 0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		case 0x36:
			DBG_EMU(20, "XOR [%d], %d ", EXPR1(), EXPR2());
			PEXPR1() ^= EXPR2();
			if (PEXPR1()^EXPR2() == 0) SET_Z() else CLR_Z();
			PC += 3;
			break;
		case 0x37:
			DBG_EMU(20, "XOR [%d+X], %d ", EXPR1(), EXPR2());
			PEXPR1X() ^= EXPR2();
			if (PEXPR1X()^EXPR2() == 0) SET_Z() else CLR_Z();
			PC += 3;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			38           5        2       ADD SP, expr 
		 */                        
		case 0x38:
			DBG_EMU(20, "ADD SP, %d ", EXPR1());
			SP += EXPR1();
			PC += 2;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			39           5        2       CMP A, expr          if (A=B)
			3A           7        2       CMP A, [expr]         Z=1
			3B           8        2       CMP A, [X+expr]      if (A<B)
			3C           8        3       CMP [expr], expr      C=1
			3D           9        3       CMP [X+expr], expr 
		 */                       
		case 0x39:
			DBG_EMU(20, "CMP A, %d ", EXPR1());
			if (A == EXPR1()) SET_Z() else CLR_Z();
			if (A < EXPR1()) SET_C() else CLR_C();
			PC += 2;
			break;
		case 0x3A:
			DBG_EMU(20, "CMP A, [%d] ", EXPR1());
			if (A == PEXPR1()) SET_Z() else CLR_Z();
			if (A < PEXPR1()) SET_C() else CLR_C();
			PC += 2;
			break;
		case 0x3B:
			DBG_EMU(20, "CMP A, [%d+X] ", EXPR1());
			if (A == PEXPR1X()) SET_Z() else CLR_Z();
			if (A < PEXPR1X()) SET_C() else CLR_C();
			PC += 2;
			break;
		case 0x3C:
			DBG_EMU(20, "CMP [%d], %d ", EXPR1(), EXPR2());
			if (PEXPR1() == EXPR2()) SET_Z() else CLR_Z();
			if (PEXPR1() < EXPR2()) SET_C() else CLR_C();
			PC += 3;
			break;
		case 0x3D:
			DBG_EMU(20, "CMP [%d+X], %d ", EXPR1(), EXPR2());
			if (PEXPR1X() == EXPR2()) SET_Z() else CLR_Z();
			if (PEXPR1X() < EXPR2()) SET_C() else CLR_C();
			PC += 3;
			break;
			
			
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			3E           10       2       MVI A, [ [expr]++ ]  Z 
			3F           10       2       MVI [ [expr]++ ], A  -
		 */                       
		case 0x3E:
			DBG_EMU(20, "MVI A, [ [%d]++ ] ", EXPR1());
			A = READ_RAM(PEXPR1()++);
			if (A==0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		case 0x3F:
			DBG_EMU(20, "MVI [ [%d]++ ], A ", EXPR1());
			READ_RAM(PEXPR1()++) = A;
			PC += 2;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			40           4        1       NOP                  -
		 */                         
		case 0x40:
			DBG_EMU(20, "NOP ");
			PC += 1;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			41           9        3       AND reg[expr], expr   Z 
			42           10       3       AND reg[X+expr], expr Z 
		 */               
		case 0x41:
			DBG_EMU(20, "AND reg[%d], %d ", EXPR1(), EXPR2());
#if defined( EMU_REG )
			READ_REG(EXPR1()) &= EXPR2();
			if (READ_REG(EXPR1())==0) SET_Z() else CLR_Z();
			// ... REG ...
#endif
			PC += 3;
			break;
		case 0x42:
			DBG_EMU(20, "AND reg[%d+X], %d ", EXPR1(), EXPR2());
#if defined( EMU_REG )
			READ_REG((uint8_t)(EXPR1()+X)) &= EXPR2();
			if (READ_REG(EXPR1())==0) SET_Z() else CLR_Z();
			// ... REG ...
#endif
			PC += 2;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			43           9        3       OR reg[expr], expr   Z 
			44           10       3       OR reg[X+expr], expr Z 
		 */               
		case 0x43:
			DBG_EMU(20, "OR reg[%d], %d ", EXPR1(), EXPR2());
#if defined( EMU_REG )
			READ_REG(EXPR1()) |= EXPR2();
			if (READ_REG(EXPR1())==0) SET_Z() else CLR_Z();
			// ... REG ...
#endif
			PC += 3;
			break;
		case 0x44:
			DBG_EMU(20, "OR reg[%d+X], %d ", EXPR1(), EXPR2());
#if defined( EMU_REG )
			READ_REG((uint8_t)(EXPR1()+X)) |= EXPR2();
			if (READ_REG(EXPR1())==0) SET_Z() else CLR_Z();
			// ... REG ...
#endif
			PC += 3;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			45           9        3       XOR reg[expr], expr    Z 
			46           10       3       XOR reg[X+expr], expr  Z 
		 */               
		case 0x45:
			DBG_EMU(20, "XOR reg[%d], %d ", EXPR1(), EXPR2());
#if defined( EMU_REG )
			READ_REG(EXPR1()) ^= EXPR2();
			if (READ_REG(EXPR1())==0) SET_Z() else CLR_Z();
			// ... REG ...
#endif
			PC += 3;
			break;
		case 0x46:
			DBG_EMU(20, "XOR reg[%d+X], %d ", EXPR1(), EXPR2());
#if defined( EMU_REG )
			READ_REG((uint8_t)(EXPR1()+X)) ^= EXPR2();
			if (READ_REG(EXPR1())==0) SET_Z() else CLR_Z();
			// ... REG ...
#endif
			PC += 3;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			47           8        3       TST [expr], expr      Z 
			48           9        3       TST [X+expr], expr    Z 
			49           9        3       TST reg[expr], expr   Z 
			4A           10       3       TST reg[X+expr], expr Z 
		 */               
		case 0x47:
			DBG_EMU(20, "TST [%d], %d ", EXPR1(), EXPR2());
			if (PEXPR1() & EXPR2() == 0) SET_Z() else CLR_Z();
			PC += 3;
			break;
		case 0x48:
			DBG_EMU(20, "TST [%d+X], %d ", EXPR1(), EXPR2());
			if (PEXPR1X() & EXPR2() == 0) SET_Z() else CLR_Z();
			PC += 3;
			break;
		case 0x49:
			DBG_EMU(20, "TST reg[%d], %d ", EXPR1(), EXPR2());
#if defined( EMU_REG )
			if (READ_REG(EXPR1()) & EXPR2() == 0) SET_Z() else CLR_Z();
#endif
			PC += 3;
			break;
		case 0x4A:
			DBG_EMU(20, "TST reg[%d+X], %d ", EXPR1(), EXPR2());
#if defined( EMU_REG )
			if (READ_REG((uint8_t)(EXPR1()+X)) & EXPR2() == 0) SET_Z() else CLR_Z();
#endif
			PC += 3;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			4B           5        1       SWAP A, X            Z 
		 */
		case 0x4B:
			DBG_EMU(20, "SWAP A, X ");
			{
				uint8_t T;
				T = X;
				X = A;
				A = T;
			}
			if (A==0) SET_Z() else CLR_Z();
			PC += 1;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			4C           7        2       SWAP A, [expr]       Z 
			4D           7        2       SWAP X, [expr]       -
			4E           5        1       SWAP A, SP           Z 
		 */
		case 0x4C:
			DBG_EMU(20, "SWAP A, [%d] ", EXPR1());
			{
				uint8_t T;
				T = A;
				A = PEXPR1();
				PEXPR1() = T;
			}
			if (A==0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		case 0x4D:
			DBG_EMU(20, "SWAP X, [%d] ", EXPR1());
			{
				uint8_t T;
				T = X;
				X = PEXPR1();
				PEXPR1() = T;
			}
			PC += 2;
			break;
		case 0x4E:
			DBG_EMU(20, "SWAP A, SP ");
			{
				uint8_t T;
				T = SP;
				SP = A;
				A = T;
			}
			if (A==0) SET_Z() else CLR_Z();
			PC += 1;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			4F           4        1       MOV X, SP 
			50           4        2       MOV A, expr          Z 
			51           5        2       MOV A, [expr]        Z
			52           6        2       MOV A, [X+expr]      Z 
			53           5        2       MOV [expr], A 
			54           6        2       MOV [X+expr], A 
			55           8        3       MOV [expr], expr 
			56           9        3       MOV [X+expr], expr 
			57           4        2       MOV X, expr
			58           6        2       MOV X, [expr]
			59           7        2       MOV X, [X+expr]
			5A           5        2       MOV [expr], X
			5B           4        1       MOV A, X             Z
			5C           4        1       MOV X, A
			5D           6        2       MOV A, reg[expr]     Z
			5E           7        2       MOV A, reg[X+expr]   Z
			5F           10       3       MOV [expr], [expr]
			60           5        2       MOV reg[expr], A
			61           6        2       MOV reg[X+expr], A
			62           8        3       MOV reg[expr], expr
			63           9        3       MOV reg[X+expr], expr
		 */
		case 0x4F:
			DBG_EMU(20, "MOV X, SP ");
			X = SP;
			PC += 1;
			break;
		case 0x50:
			DBG_EMU(20, "MOV A, %d ", EXPR1());
			A = EXPR1();
			if (A==0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		case 0x51:
			DBG_EMU(20, "MOV A, [%d] ", EXPR1());
			A = PEXPR1();
			if (A==0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		case 0x52:
			DBG_EMU(20, "MOV A, [%d+X] ", EXPR1());
			A = PEXPR1X();
			if (A==0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		case 0x53:
			DBG_EMU(20, "MOV [%d], A ", EXPR1());
			PEXPR1() = A;
			PC += 2;
			break;
		case 0x54:
			DBG_EMU(20, "MOV [%d+X], A ", EXPR1());
			PEXPR1X() = A;
			PC += 2;
			break;
		case 0x55:
			DBG_EMU(20, "MOV [%d], %d ", EXPR1(), EXPR2());
			PEXPR1() = EXPR2();
			PC += 3;
			break;
		case 0x56:
			DBG_EMU(20, "MOV [%d+X], %d ", EXPR1(), EXPR2());
			PEXPR1X() = EXPR2();
			PC += 3;
			break;
		case 0x57:
			DBG_EMU(20, "MOV X, %d ", EXPR1());
			X = EXPR1();
			PC += 2;
			break;
		case 0x58:
			DBG_EMU(20, "MOV X, [%d] ", EXPR1());
			X = PEXPR1();
			PC += 2;
			break;
		case 0x59:
			DBG_EMU(20, "MOV X, [%d+X] ", EXPR1());
			X = PEXPR1X();
			PC += 2;
			break;
		case 0x5A:
			DBG_EMU(20, "MOV %d, X ", EXPR1());
			PEXPR1() = X;
			PC += 2;
			break;
		case 0x5B:
			DBG_EMU(20, "MOV A, X ");
			A = X;
			if (A==0) SET_Z() else CLR_Z();
			PC += 1;
			break;
		case 0x5C:
			DBG_EMU(20, "MOV X, A ");
			X = A;
			PC += 1;
			break;
		case 0x5D:
			DBG_EMU(20, "MOV A, reg[%d] ", EXPR1());
#if defined( EMU_REG )
			A = READ_REG(EXPR1());
			if (A==0) SET_Z() else CLR_Z();
			// ... REG ...
#endif
			PC += 2;
			break;
		case 0x5E:
			DBG_EMU(20, "MOV A, reg[%d+X] ", EXPR1());
#if defined( EMU_REG )
			A = READ_REG((uint8_t)(EXPR1()+X));
			if (A==0) SET_Z() else CLR_Z();
			// ... REG ...
#endif
			PC += 2;
			break;
		case 0x5F:
			DBG_EMU(20, "MOV [%d], [%d] ", EXPR1(), EXPR2());
			PEXPR1() = PEXPR2();
			PC += 3;
			break;
		case 0x60:
			DBG_EMU(20, "MOV reg[%d], A ", EXPR1());
#if defined( EMU_REG )
			READ_REG(EXPR1()) = A;
			// ... REG ...
#endif
			PC += 2;
			break;
		case 0x61:
			DBG_EMU(20, "MOV reg[%d+X], A ", EXPR1());
#if defined( EMU_REG )
			READ_REG((uint8_t)(EXPR1()+X)) = A;
			// ... REG ...
#endif
			PC += 2;
			break;
		case 0x62:
			DBG_EMU(20, "MOV reg[%d], %d ", EXPR1(), EXPR2());
#if defined( EMU_REG )
			READ_REG(EXPR1()) = EXPR2();
			// ... REG ...
#endif
			PC += 3;
			break;
		case 0x63:
			DBG_EMU(20, "MOV reg[%d+X], %d ", EXPR1(), EXPR2());
#if defined( EMU_REG )
			READ_REG((uint8_t)(EXPR1()+X)) = EXPR2();
			// ... REG ...
#endif
			PC += 3;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			64           4        1       ASL A                C, Z
			65           7        2       ASL [expr]           C, Z
			66           8        2       ASL [X+expr]         C, Z
		 */
		case 0x64:
			DBG_EMU(20, "ASL A ");
			if (A & 0x80) SET_C() else CLR_C();
			A <<= 1;
			if (A==0) SET_Z() else CLR_Z();
			PC += 1;
			break;
		case 0x65:
			DBG_EMU(20, "ASL [%d] ", EXPR1());
			if (PEXPR1() & 0x80) SET_C() else CLR_C();
			PEXPR1() <<= 1;
			if (PEXPR1()==0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		case 0x66:
			DBG_EMU(20, "ASL [%d+X] ", EXPR1());
			if (PEXPR1X() & 0x80) SET_C() else CLR_C();
			PEXPR1X() <<= 1;
			if (PEXPR1X()==0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			67           4        1       ASR A                C, Z
			68           7        2       ASR [expr]           C, Z
			69           8        2       ASR [X+expr]         C, Z
		 */
		case 0x67:
			DBG_EMU(20, "ASR A ");
			if (A & 0x01) SET_C() else CLR_C();
			A >>= 1;
			if (A==0) SET_Z() else CLR_Z();
			PC += 1;
			break;
		case 0x68:
			DBG_EMU(20, "ASR [%d] ", EXPR1());
			if (PEXPR1() & 0x01) SET_C() else CLR_C();
			PEXPR1() >>= 1;
			if (PEXPR1()==0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		case 0x69:
			DBG_EMU(20, "ASR [%d+X] ", EXPR1());
			if (PEXPR1X() & 0x01) SET_C() else CLR_C();
			PEXPR1X() >>= 1;
			if (PEXPR1X()==0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			6A           4        1       RLC A                C, Z
			6B           7        2       RLC [expr]           C, Z
			6C           8        2       RLC [X+expr]         C, Z
		 */               
		case 0x6A:
			DBG_EMU(20, "RLC A ");
			if (A & 0x80) SET_C() else CLR_C();
			A <<= 1;
			A |= IS_C();
			if (A==0) SET_Z() else CLR_Z();
			PC += 1;
			break;
		case 0x6B:
			DBG_EMU(20, "RLC [%d] ", EXPR1());
			if (PEXPR1() & 0x80) SET_C() else CLR_C();
			PEXPR1() <<= 1;
			PEXPR1()  |= IS_C();
			if (PEXPR1()==0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		case 0x6C:
			DBG_EMU(20, "RLC [%d+X] ", EXPR1());
			if (PEXPR1X() & 0x80) SET_C() else CLR_C();
			PEXPR1X() <<= 1;
			PEXPR1X()  |= IS_C();
			if (PEXPR1X()==0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			6D           4        1       RRC A C,             Z
			6E           7        2       RRC [expr]           C, Z
			6F           8        2       RRC [X+expr]         C, Z
		 */              
		case 0x6D:
			DBG_EMU(20, "RRC A ");
			if (A & 0x80) SET_C() else CLR_C();
			A <<= 1;
			A |= IS_C();
			if (A==0) SET_Z() else CLR_Z();
			PC += 1;
			break;
		case 0x6E:
			DBG_EMU(20, "RRC [%d] ", EXPR1());
			if (PEXPR1() & 0x80) SET_C() else CLR_C();
			PEXPR1() <<= 1;
			PEXPR1() |= IS_C();
			if (PEXPR1()==0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		case 0x6F:
			DBG_EMU(20, "RRC [%d+X] ", EXPR1());
			if (PEXPR1X() & 0x80) SET_C() else CLR_C();
			PEXPR1X() <<= 1;
			PEXPR1X() |= IS_C();
			if (PEXPR1X()==0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			70           4        2       AND F, expr          C, Z
		 */              
		case 0x70:
			DBG_EMU(20, "AND F, %02x", EXPR1());
			F &= EXPR1();
			PC += 2;
			if (GET_PGMODE() + 1 > EMU_RAM_BANKS)
			{
#if 1
				DBG_EMU(15, "| Return VMRET_RAMBANK_ERR;");
				DBG_EMU(0, "\n");
				return VMRET_RAMBANK_ERR;
#endif
			}
			else
			{
				ram = ram_img[GET_PGMODE()];
			}
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			71           4        2       OR F, expr           C, Z
		 */              
		case 0x71:
			DBG_EMU(20, "OR F, %02x", EXPR1());
			F |= EXPR1();
			PC += 2;
			if (GET_PGMODE() + 1 > EMU_RAM_BANKS)
			{
#if 1
				DBG_EMU(15, "| Return VMRET_RAMBANK_ERR;");
				DBG_EMU(0, "\n");
				return VMRET_RAMBANK_ERR;
#endif
			}
			else
			{
				ram = ram_img[GET_PGMODE()];
			}
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			72           4        2       XOR F, expr          C, Z
		 */
		case 0x72:
			DBG_EMU(20, "XOR F, %02x", EXPR1());
			F ^= EXPR1();
			PC += 2;
			if (GET_PGMODE() + 1 > EMU_RAM_BANKS)
			{
#if 1
				DBG_EMU(15, "| Return VMRET_RAMBANK_ERR;");
				DBG_EMU(0, "\n");
				return VMRET_RAMBANK_ERR;
#endif
			}
			else
			{
				ram = ram_img[GET_PGMODE()];
			}
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			73           4        1       CPL A                Z
		 */
		case 0x73:
			DBG_EMU(20, "CPL A ");
			A = ~A;
			if (A==0) SET_Z() else CLR_Z();
			PC += 1;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			74           4        1       INC A                C, Z
			75           4        1       INC X                C, Z
			76           7        2       INC [expr]           C, Z
			77           8        2       INC [X+expr]         C, Z
		 */
		case 0x74:
			DBG_EMU(20, "INC A ");
			if (++A) { CLR_C(); CLR_Z(); } else { SET_C(); SET_Z(); } 
			PC += 1;
			break;
		case 0x75:
			DBG_EMU(20, "INC X ");
			if (++X) { CLR_C(); CLR_Z(); } else { SET_C(); SET_Z(); } 
			PC += 1;
			break;
		case 0x76:
			DBG_EMU(20, "INC [%d] ", EXPR1());
			if (++PEXPR1()) { CLR_C(); CLR_Z(); } else { SET_C(); SET_Z(); } 
			PC += 2;
			break;
		case 0x77:
			DBG_EMU(20, "INC [%d+X] ", EXPR1());
			if (++PEXPR1X()) { CLR_C(); CLR_Z(); } else { SET_C(); SET_Z(); } 
			PC += 2;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			78           4        1       DEC A                C, Z
			79           4        1       DEC X                C, Z
			7A           7        2       DEC [expr]           C, Z
			7B           8        2       DEC [X+expr]         C, Z
		 */                                           
		case 0x78:
			DBG_EMU(20, "DEC A ");
			if (A == 0) SET_C() else CLR_C();
			if (--A == 0) SET_Z() else CLR_Z();
			PC += 1;
			break;
		case 0x79:
			DBG_EMU(20, "DEC X ");
			if (X == 0) SET_C() else CLR_C();
			if (--X == 0) SET_Z() else CLR_Z();
			PC += 1;
			break;
		case 0x7A:
			DBG_EMU(20, "DEC [%d] ", EXPR1());
			if (PEXPR1() == 0) SET_C() else CLR_C();
			if (--PEXPR1() == 0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		case 0x7B:
			DBG_EMU(20, "DEC [%d+X] ", EXPR1());
			if (PEXPR1X() == 0) SET_C() else CLR_C();
			if (--PEXPR1X() == 0) SET_Z() else CLR_Z();
			PC += 2;
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			7C           13       3       LCALL
		 */
		case 0x7C:
			DBG_EMU(20, "LCALL %04x ", ABS_ADDR());
			PUSH(SP) = HB(PC+3);
			PUSH(SP) = LB(PC+3);
			PC = ABS_ADDR();
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			7D           7        3       LJMP
		 */
		case 0x7D:
			DBG_EMU(20, "LJMP %04x ", ABS_ADDR());
			PC = ABS_ADDR();
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			7E           10       1       RETI                 C, Z
		 */
		case 0x7E:
			DBG_EMU(20, "RETI ");
			// Int...
#if 1
			DBG_EMU(15, "| Return VMRET_RETI;");
			DBG_EMU(0, "\n");
			return VMRET_RETI;
#else
			PC  =  POP(SP);
			PC |= ((uint16_t)POP(SP) << 8);
#endif
			break;
		
		/*
			Opcode Hex | Cycles | Bytes | Instruction Format | Flags
			7F           8        1       RET
		 */
		case 0x7F:
			DBG_EMU(20, "RET        ");
			PC  =  POP(SP);
			PC |= ((uint16_t)POP(SP) << 8);
			break;
			
		default:
			// WTF?
			DBG_EMU(15, "| return VMRET_UNDEF;");
			DBG_EMU(0, "\n");
			return VMRET_UNDEF;
		}
	}
	
	DBG_EMU(50, "| REGS: A=%02x X=%02x F=%02x | SP = %02x | Return: VMRET_OK;", A, X, F, SP);
	DBG_EMU(0, "\n");
	
	return VMRET_OK;
}

#ifdef EMU_TEST_1

typedef unsigned char BYTE;
typedef unsigned short WORD;
typedef unsigned long DWORD;

DWORD f2b(const char *name, BYTE **ppbuffer, DWORD *psize);

int main(int n, char **arg)
{
	BYTE *pbBuffer;
	DWORD dwLen;
	int ret;
	
	if (n != 2) 
	{
		printf("error 1!\n");
		return 1;
	}
	
	if (f2b(arg[1], &pbBuffer, &dwLen))
	{
		printf("error 2\n");
		return 1;
	}
	
	cy803_init(pbBuffer);
	
	while ((ret = cy803_step()) == 0);
	
	printf("ret = %d\n", ret);

	return 0;
}

/* File to buffer */
DWORD f2b(const char *name, BYTE **ppbuffer, DWORD *psize)
{
	FILE *f;
	DWORD bsize = 0;
	*psize = 0x00;
	if ((f = fopen(name, "rb")) == 0)
	{
		return 1;
	}
	*ppbuffer = (BYTE*)malloc(1024*sizeof(BYTE));
	do
	{
		bsize = fread(&((*ppbuffer)[*psize]), 1, 1024, f);
		*psize += bsize;
		*ppbuffer = (BYTE*)realloc(*ppbuffer, (*psize + 1024)*sizeof(BYTE));
	}
	while (bsize == 1024);
	fclose(f);
	return 0;
}
#endif

#ifdef EMU_TEST_2

typedef unsigned char BYTE;
typedef unsigned short WORD;
typedef unsigned long DWORD;

DWORD f2b(const char *name, BYTE **ppbuffer, DWORD *psize);

int main(int n, char **arg)
{
	BYTE *pbBuffer;
	int ret;
	long i;
	
	pbBuffer = malloc(0x10000*sizeof(BYTE));
	
	for (i = 0; i < 0x100; i++)
	{
		pbBuffer[0] = (BYTE)i;
		pbBuffer[1] = 0xAA;
		pbBuffer[2] = 0xAA;
		
		cy803_init(pbBuffer);
		ret = cy803_step();
	}
	
	return 0;
}
#endif

#ifdef EMU_TEST_3
#endif
